
/*
 * Structures
 */
typedef struct transaction_s
{
    ssize_t i_size;
    unsigned char *p_data;

} transaction_t;

/*
 * Prototypes
 */
int EcrireTransaction( char *psz_file, unsigned char *p_data, ssize_t i_size );
transaction_t * LireTransaction( char *psz_file );

transaction_t * MallocTransaction( ssize_t i_size );
transaction_t * ReallocTransaction( transaction_t *p_trans, ssize_t i_size );
transaction_t * AppendToTransaction( transaction_t *p_trans, unsigned char *p_data, ssize_t i_size );
void FreeTransaction( transaction_t *p_trans );

/*
 * Defines
 */
#define READ_ONCE 32 /* bytes to read at the same time */
#define DATA_ALIGN 32 /* transaction alignment */
#define I_ERROR 1 /* error flag */
#define I_EOF 2 /* EOF flag */

/*
 * Macros used by LireTransaction()
 */

/* READCHAR : read a char from descriptor i_fd to *P_CHAR */
#define READCHAR( P_CHAR ) \
if( i_buf_len > 0 ) \
{ \
    *((char*)P_CHAR) = p_buf[ i_buf_pos ]; \
    i_buf_len--; \
    i_buf_pos++; \
} \
else \
{ \
    i_buf_len = read( i_fd, p_buf, READ_ONCE ); \
    i_buf_pos = 0; \
    if( i_buf_len < 0 ) \
    { \
        ERROR( "LireTransaction: read error" ); \
        goto error; \
    } \
    else if( i_buf_len == 0 ) \
    { \
        ERROR( "LireTransaction: EOF reached" ); \
        goto error; \
    } \
    else \
    { \
        *((char*)P_CHAR) = p_buf[ i_buf_pos ]; \
        i_buf_len--; \
        i_buf_pos++; \
    } \
}

/* ADDCHAR: append CHAR to p_trans */
#define ADDCHAR( CHAR ) \
do { \
    p_trans = AppendToTransaction( p_trans, (unsigned char *)(&(CHAR)), 1 ); \
    if( p_trans == NULL ) \
    { \
        ERROR( "LireTransaction: realloc error" ); \
        goto error; \
    } \
} while( 0 );

